def basicrst(arg1, arg2):
    """

    This is a title
    ===============

    This is a section
    -----------------

    1. primo elemento elenco numerato *corsivo*;
    2. secondo elemento elenco numerato **grassetto**:

       * sottoelenco puntato 1 
       * sottoelenco puntato 2

    3. codice ``print(100)``


    This is a subsection
    ~~~~~~~~~~~~~~~~~~~~

    .. Questo è un commento.

    Questo è un `link a una pagina <https://www.google.it/>`_

    definizione1
     Definition of the term, which must be indented

    definizione2
     Description.

    Blocchi di codice sono introdotti da doppio due punti (e indentati)::
    
        ls asd
        sudo rm -rf /


    """
    return arg1 + arg2
